"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2013 The Chromium Authors. All rights reserved.
Use of this source code is governed by a BSD-style license that can be
found in the LICENSE file.

MCS protocol for communication between Chrome client and Mobile Connection
Server .
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class HeartbeatPing(google.protobuf.message.Message):
    """
    Common fields/comments:

    stream_id: no longer sent by server, each side keeps a counter
    last_stream_id_received: sent only if a packet was received since last time
    a last_stream was sent
    status: new bitmask including the 'idle' as bit 0.

    *
    TAG: 0
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STREAM_ID_FIELD_NUMBER: builtins.int
    LAST_STREAM_ID_RECEIVED_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    stream_id: builtins.int
    last_stream_id_received: builtins.int
    status: builtins.int
    def __init__(
        self,
        *,
        stream_id: builtins.int | None = ...,
        last_stream_id_received: builtins.int | None = ...,
        status: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "last_stream_id_received",
            b"last_stream_id_received",
            "status",
            b"status",
            "stream_id",
            b"stream_id",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "last_stream_id_received",
            b"last_stream_id_received",
            "status",
            b"status",
            "stream_id",
            b"stream_id",
        ],
    ) -> None: ...

global___HeartbeatPing = HeartbeatPing

@typing_extensions.final
class HeartbeatAck(google.protobuf.message.Message):
    """*
    TAG: 1
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STREAM_ID_FIELD_NUMBER: builtins.int
    LAST_STREAM_ID_RECEIVED_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    stream_id: builtins.int
    last_stream_id_received: builtins.int
    status: builtins.int
    def __init__(
        self,
        *,
        stream_id: builtins.int | None = ...,
        last_stream_id_received: builtins.int | None = ...,
        status: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "last_stream_id_received",
            b"last_stream_id_received",
            "status",
            b"status",
            "stream_id",
            b"stream_id",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "last_stream_id_received",
            b"last_stream_id_received",
            "status",
            b"status",
            "stream_id",
            b"stream_id",
        ],
    ) -> None: ...

global___HeartbeatAck = HeartbeatAck

@typing_extensions.final
class ErrorInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CODE_FIELD_NUMBER: builtins.int
    MESSAGE_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    EXTENSION_FIELD_NUMBER: builtins.int
    code: builtins.int
    message: builtins.str
    type: builtins.str
    @property
    def extension(self) -> global___Extension: ...
    def __init__(
        self,
        *,
        code: builtins.int | None = ...,
        message: builtins.str | None = ...,
        type: builtins.str | None = ...,
        extension: global___Extension | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "code",
            b"code",
            "extension",
            b"extension",
            "message",
            b"message",
            "type",
            b"type",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "code",
            b"code",
            "extension",
            b"extension",
            "message",
            b"message",
            "type",
            b"type",
        ],
    ) -> None: ...

global___ErrorInfo = ErrorInfo

@typing_extensions.final
class Setting(google.protobuf.message.Message):
    """MobileSettings class.
    "u:f", "u:b", "u:s" - multi user devices reporting foreground, background
    and stopped users.
    hbping: heatbeat ping interval
    rmq2v: include explicit stream IDs
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    name: builtins.str
    value: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        value: builtins.str | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["name", b"name", "value", b"value"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["name", b"name", "value", b"value"]
    ) -> None: ...

global___Setting = Setting

@typing_extensions.final
class HeartbeatStat(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IP_FIELD_NUMBER: builtins.int
    TIMEOUT_FIELD_NUMBER: builtins.int
    INTERVAL_MS_FIELD_NUMBER: builtins.int
    ip: builtins.str
    timeout: builtins.bool
    interval_ms: builtins.int
    def __init__(
        self,
        *,
        ip: builtins.str | None = ...,
        timeout: builtins.bool | None = ...,
        interval_ms: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "interval_ms", b"interval_ms", "ip", b"ip", "timeout", b"timeout"
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "interval_ms", b"interval_ms", "ip", b"ip", "timeout", b"timeout"
        ],
    ) -> None: ...

global___HeartbeatStat = HeartbeatStat

@typing_extensions.final
class HeartbeatConfig(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UPLOAD_STAT_FIELD_NUMBER: builtins.int
    IP_FIELD_NUMBER: builtins.int
    INTERVAL_MS_FIELD_NUMBER: builtins.int
    upload_stat: builtins.bool
    ip: builtins.str
    interval_ms: builtins.int
    def __init__(
        self,
        *,
        upload_stat: builtins.bool | None = ...,
        ip: builtins.str | None = ...,
        interval_ms: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "interval_ms", b"interval_ms", "ip", b"ip", "upload_stat", b"upload_stat"
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "interval_ms", b"interval_ms", "ip", b"ip", "upload_stat", b"upload_stat"
        ],
    ) -> None: ...

global___HeartbeatConfig = HeartbeatConfig

@typing_extensions.final
class ClientEvent(google.protobuf.message.Message):
    """ClientEvents are used to inform the server of failed and successful
    connections.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Type:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _TypeEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[
            ClientEvent._Type.ValueType
        ],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: ClientEvent._Type.ValueType  # 0
        DISCARDED_EVENTS: ClientEvent._Type.ValueType  # 1
        """Count of discarded events if the buffer filled up and was trimmed."""
        FAILED_CONNECTION: ClientEvent._Type.ValueType  # 2
        """Failed connection event: the connection failed to be established or we
        had a login error.
        """
        SUCCESSFUL_CONNECTION: ClientEvent._Type.ValueType  # 3
        """Successful connection event: information about the last successful
        connection, including the time at which it was established.
        """

    class Type(_Type, metaclass=_TypeEnumTypeWrapper): ...
    UNKNOWN: ClientEvent.Type.ValueType  # 0
    DISCARDED_EVENTS: ClientEvent.Type.ValueType  # 1
    """Count of discarded events if the buffer filled up and was trimmed."""
    FAILED_CONNECTION: ClientEvent.Type.ValueType  # 2
    """Failed connection event: the connection failed to be established or we
    had a login error.
    """
    SUCCESSFUL_CONNECTION: ClientEvent.Type.ValueType  # 3
    """Successful connection event: information about the last successful
    connection, including the time at which it was established.
    """

    TYPE_FIELD_NUMBER: builtins.int
    NUMBER_DISCARDED_EVENTS_FIELD_NUMBER: builtins.int
    NETWORK_TYPE_FIELD_NUMBER: builtins.int
    TIME_CONNECTION_STARTED_MS_FIELD_NUMBER: builtins.int
    TIME_CONNECTION_ENDED_MS_FIELD_NUMBER: builtins.int
    ERROR_CODE_FIELD_NUMBER: builtins.int
    TIME_CONNECTION_ESTABLISHED_MS_FIELD_NUMBER: builtins.int
    type: global___ClientEvent.Type.ValueType
    """Common fields [1-99]"""
    number_discarded_events: builtins.int
    """Fields for DISCARDED_EVENTS messages [100-199]"""
    network_type: builtins.int
    """Fields for FAILED_CONNECTION and SUCCESSFUL_CONNECTION messages [200-299]
    Network type is a value in net::NetworkChangeNotifier::ConnectionType.
    """
    time_connection_started_ms: builtins.int
    time_connection_ended_ms: builtins.int
    error_code: builtins.int
    """Error code should be a net::Error value."""
    time_connection_established_ms: builtins.int
    """Fields for SUCCESSFUL_CONNECTION messages [300-399]"""
    def __init__(
        self,
        *,
        type: global___ClientEvent.Type.ValueType | None = ...,
        number_discarded_events: builtins.int | None = ...,
        network_type: builtins.int | None = ...,
        time_connection_started_ms: builtins.int | None = ...,
        time_connection_ended_ms: builtins.int | None = ...,
        error_code: builtins.int | None = ...,
        time_connection_established_ms: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "error_code",
            b"error_code",
            "network_type",
            b"network_type",
            "number_discarded_events",
            b"number_discarded_events",
            "time_connection_ended_ms",
            b"time_connection_ended_ms",
            "time_connection_established_ms",
            b"time_connection_established_ms",
            "time_connection_started_ms",
            b"time_connection_started_ms",
            "type",
            b"type",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "error_code",
            b"error_code",
            "network_type",
            b"network_type",
            "number_discarded_events",
            b"number_discarded_events",
            "time_connection_ended_ms",
            b"time_connection_ended_ms",
            "time_connection_established_ms",
            b"time_connection_established_ms",
            "time_connection_started_ms",
            b"time_connection_started_ms",
            "type",
            b"type",
        ],
    ) -> None: ...

global___ClientEvent = ClientEvent

@typing_extensions.final
class LoginRequest(google.protobuf.message.Message):
    """*
    TAG: 2
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _AuthService:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _AuthServiceEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[
            LoginRequest._AuthService.ValueType
        ],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        ANDROID_ID: LoginRequest._AuthService.ValueType  # 2

    class AuthService(_AuthService, metaclass=_AuthServiceEnumTypeWrapper): ...
    ANDROID_ID: LoginRequest.AuthService.ValueType  # 2

    ID_FIELD_NUMBER: builtins.int
    DOMAIN_FIELD_NUMBER: builtins.int
    USER_FIELD_NUMBER: builtins.int
    RESOURCE_FIELD_NUMBER: builtins.int
    AUTH_TOKEN_FIELD_NUMBER: builtins.int
    DEVICE_ID_FIELD_NUMBER: builtins.int
    LAST_RMQ_ID_FIELD_NUMBER: builtins.int
    SETTING_FIELD_NUMBER: builtins.int
    RECEIVED_PERSISTENT_ID_FIELD_NUMBER: builtins.int
    ADAPTIVE_HEARTBEAT_FIELD_NUMBER: builtins.int
    HEARTBEAT_STAT_FIELD_NUMBER: builtins.int
    USE_RMQ2_FIELD_NUMBER: builtins.int
    ACCOUNT_ID_FIELD_NUMBER: builtins.int
    AUTH_SERVICE_FIELD_NUMBER: builtins.int
    NETWORK_TYPE_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    CLIENT_EVENT_FIELD_NUMBER: builtins.int
    id: builtins.str
    """Must be present ( proto required ), may be empty"""
    domain: builtins.str
    """string.
    mcs.android.com.
    """
    user: builtins.str
    """Decimal android ID"""
    resource: builtins.str
    auth_token: builtins.str
    """Secret"""
    device_id: builtins.str
    """Format is: android-HEX_DEVICE_ID
    The user is the decimal value.
    """
    last_rmq_id: builtins.int
    """RMQ1 - no longer used"""
    @property
    def setting(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        global___Setting
    ]: ...
    @property
    def received_persistent_id(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """optional int32 compress = 9;"""
    adaptive_heartbeat: builtins.bool
    """Replaced by "rmq2v" setting
    optional bool include_stream_ids = 11;
    """
    @property
    def heartbeat_stat(self) -> global___HeartbeatStat: ...
    use_rmq2: builtins.bool
    """Must be true."""
    account_id: builtins.int
    auth_service: global___LoginRequest.AuthService.ValueType
    """ANDROID_ID = 2"""
    network_type: builtins.int
    status: builtins.int
    @property
    def client_event(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        global___ClientEvent
    ]:
        """Events recorded on the client after the last successful connection."""
    def __init__(
        self,
        *,
        id: builtins.str | None = ...,
        domain: builtins.str | None = ...,
        user: builtins.str | None = ...,
        resource: builtins.str | None = ...,
        auth_token: builtins.str | None = ...,
        device_id: builtins.str | None = ...,
        last_rmq_id: builtins.int | None = ...,
        setting: collections.abc.Iterable[global___Setting] | None = ...,
        received_persistent_id: collections.abc.Iterable[builtins.str] | None = ...,
        adaptive_heartbeat: builtins.bool | None = ...,
        heartbeat_stat: global___HeartbeatStat | None = ...,
        use_rmq2: builtins.bool | None = ...,
        account_id: builtins.int | None = ...,
        auth_service: global___LoginRequest.AuthService.ValueType | None = ...,
        network_type: builtins.int | None = ...,
        status: builtins.int | None = ...,
        client_event: collections.abc.Iterable[global___ClientEvent] | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "account_id",
            b"account_id",
            "adaptive_heartbeat",
            b"adaptive_heartbeat",
            "auth_service",
            b"auth_service",
            "auth_token",
            b"auth_token",
            "device_id",
            b"device_id",
            "domain",
            b"domain",
            "heartbeat_stat",
            b"heartbeat_stat",
            "id",
            b"id",
            "last_rmq_id",
            b"last_rmq_id",
            "network_type",
            b"network_type",
            "resource",
            b"resource",
            "status",
            b"status",
            "use_rmq2",
            b"use_rmq2",
            "user",
            b"user",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "account_id",
            b"account_id",
            "adaptive_heartbeat",
            b"adaptive_heartbeat",
            "auth_service",
            b"auth_service",
            "auth_token",
            b"auth_token",
            "client_event",
            b"client_event",
            "device_id",
            b"device_id",
            "domain",
            b"domain",
            "heartbeat_stat",
            b"heartbeat_stat",
            "id",
            b"id",
            "last_rmq_id",
            b"last_rmq_id",
            "network_type",
            b"network_type",
            "received_persistent_id",
            b"received_persistent_id",
            "resource",
            b"resource",
            "setting",
            b"setting",
            "status",
            b"status",
            "use_rmq2",
            b"use_rmq2",
            "user",
            b"user",
        ],
    ) -> None: ...

global___LoginRequest = LoginRequest

@typing_extensions.final
class LoginResponse(google.protobuf.message.Message):
    """*
    TAG: 3
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    JID_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    SETTING_FIELD_NUMBER: builtins.int
    STREAM_ID_FIELD_NUMBER: builtins.int
    LAST_STREAM_ID_RECEIVED_FIELD_NUMBER: builtins.int
    HEARTBEAT_CONFIG_FIELD_NUMBER: builtins.int
    SERVER_TIMESTAMP_FIELD_NUMBER: builtins.int
    id: builtins.str
    jid: builtins.str
    """Not used."""
    @property
    def error(self) -> global___ErrorInfo:
        """Null if login was ok."""
    @property
    def setting(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        global___Setting
    ]: ...
    stream_id: builtins.int
    last_stream_id_received: builtins.int
    """Should be "1" """
    @property
    def heartbeat_config(self) -> global___HeartbeatConfig: ...
    server_timestamp: builtins.int
    """used by the client to synchronize with the server timestamp."""
    def __init__(
        self,
        *,
        id: builtins.str | None = ...,
        jid: builtins.str | None = ...,
        error: global___ErrorInfo | None = ...,
        setting: collections.abc.Iterable[global___Setting] | None = ...,
        stream_id: builtins.int | None = ...,
        last_stream_id_received: builtins.int | None = ...,
        heartbeat_config: global___HeartbeatConfig | None = ...,
        server_timestamp: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "error",
            b"error",
            "heartbeat_config",
            b"heartbeat_config",
            "id",
            b"id",
            "jid",
            b"jid",
            "last_stream_id_received",
            b"last_stream_id_received",
            "server_timestamp",
            b"server_timestamp",
            "stream_id",
            b"stream_id",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "error",
            b"error",
            "heartbeat_config",
            b"heartbeat_config",
            "id",
            b"id",
            "jid",
            b"jid",
            "last_stream_id_received",
            b"last_stream_id_received",
            "server_timestamp",
            b"server_timestamp",
            "setting",
            b"setting",
            "stream_id",
            b"stream_id",
        ],
    ) -> None: ...

global___LoginResponse = LoginResponse

@typing_extensions.final
class StreamErrorStanza(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_FIELD_NUMBER: builtins.int
    TEXT_FIELD_NUMBER: builtins.int
    type: builtins.str
    text: builtins.str
    def __init__(
        self,
        *,
        type: builtins.str | None = ...,
        text: builtins.str | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["text", b"text", "type", b"type"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["text", b"text", "type", b"type"]
    ) -> None: ...

global___StreamErrorStanza = StreamErrorStanza

@typing_extensions.final
class Close(google.protobuf.message.Message):
    """*
    TAG: 4
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___Close = Close

@typing_extensions.final
class Extension(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    id: builtins.int
    """12: SelectiveAck
    13: StreamAck
    """
    data: builtins.bytes
    def __init__(
        self,
        *,
        id: builtins.int | None = ...,
        data: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["data", b"data", "id", b"id"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["data", b"data", "id", b"id"]
    ) -> None: ...

global___Extension = Extension

@typing_extensions.final
class IqStanza(google.protobuf.message.Message):
    """*
    TAG: 7
    IqRequest must contain a single extension.  IqResponse may contain 0 or 1
    extensions.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _IqType:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _IqTypeEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[
            IqStanza._IqType.ValueType
        ],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        GET: IqStanza._IqType.ValueType  # 0
        SET: IqStanza._IqType.ValueType  # 1
        RESULT: IqStanza._IqType.ValueType  # 2
        IQ_ERROR: IqStanza._IqType.ValueType  # 3

    class IqType(_IqType, metaclass=_IqTypeEnumTypeWrapper): ...
    GET: IqStanza.IqType.ValueType  # 0
    SET: IqStanza.IqType.ValueType  # 1
    RESULT: IqStanza.IqType.ValueType  # 2
    IQ_ERROR: IqStanza.IqType.ValueType  # 3

    RMQ_ID_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    FROM_FIELD_NUMBER: builtins.int
    TO_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    EXTENSION_FIELD_NUMBER: builtins.int
    PERSISTENT_ID_FIELD_NUMBER: builtins.int
    STREAM_ID_FIELD_NUMBER: builtins.int
    LAST_STREAM_ID_RECEIVED_FIELD_NUMBER: builtins.int
    ACCOUNT_ID_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    rmq_id: builtins.int
    type: global___IqStanza.IqType.ValueType
    id: builtins.str
    to: builtins.str
    @property
    def error(self) -> global___ErrorInfo: ...
    @property
    def extension(self) -> global___Extension:
        """Only field used in the 38+ protocol (besides common last_stream_id_received, status, rmq_id)"""
    persistent_id: builtins.str
    stream_id: builtins.int
    last_stream_id_received: builtins.int
    account_id: builtins.int
    status: builtins.int
    def __init__(
        self,
        *,
        rmq_id: builtins.int | None = ...,
        type: global___IqStanza.IqType.ValueType | None = ...,
        id: builtins.str | None = ...,
        to: builtins.str | None = ...,
        error: global___ErrorInfo | None = ...,
        extension: global___Extension | None = ...,
        persistent_id: builtins.str | None = ...,
        stream_id: builtins.int | None = ...,
        last_stream_id_received: builtins.int | None = ...,
        account_id: builtins.int | None = ...,
        status: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "account_id",
            b"account_id",
            "error",
            b"error",
            "extension",
            b"extension",
            "from",
            b"from",
            "id",
            b"id",
            "last_stream_id_received",
            b"last_stream_id_received",
            "persistent_id",
            b"persistent_id",
            "rmq_id",
            b"rmq_id",
            "status",
            b"status",
            "stream_id",
            b"stream_id",
            "to",
            b"to",
            "type",
            b"type",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "account_id",
            b"account_id",
            "error",
            b"error",
            "extension",
            b"extension",
            "from",
            b"from",
            "id",
            b"id",
            "last_stream_id_received",
            b"last_stream_id_received",
            "persistent_id",
            b"persistent_id",
            "rmq_id",
            b"rmq_id",
            "status",
            b"status",
            "stream_id",
            b"stream_id",
            "to",
            b"to",
            "type",
            b"type",
        ],
    ) -> None: ...

global___IqStanza = IqStanza

@typing_extensions.final
class AppData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    key: builtins.str
    value: builtins.str
    def __init__(
        self,
        *,
        key: builtins.str | None = ...,
        value: builtins.str | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]
    ) -> None: ...

global___AppData = AppData

@typing_extensions.final
class DataMessageStanza(google.protobuf.message.Message):
    """*
    TAG: 8
    Not used.
    optional int64 rmq_id = 1;
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    FROM_FIELD_NUMBER: builtins.int
    TO_FIELD_NUMBER: builtins.int
    CATEGORY_FIELD_NUMBER: builtins.int
    TOKEN_FIELD_NUMBER: builtins.int
    APP_DATA_FIELD_NUMBER: builtins.int
    FROM_TRUSTED_SERVER_FIELD_NUMBER: builtins.int
    PERSISTENT_ID_FIELD_NUMBER: builtins.int
    STREAM_ID_FIELD_NUMBER: builtins.int
    LAST_STREAM_ID_RECEIVED_FIELD_NUMBER: builtins.int
    REG_ID_FIELD_NUMBER: builtins.int
    DEVICE_USER_ID_FIELD_NUMBER: builtins.int
    TTL_FIELD_NUMBER: builtins.int
    SENT_FIELD_NUMBER: builtins.int
    QUEUED_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    RAW_DATA_FIELD_NUMBER: builtins.int
    IMMEDIATE_ACK_FIELD_NUMBER: builtins.int
    id: builtins.str
    """This is the message ID, set by client, DMP.9 (message_id)"""
    to: builtins.str
    """Part of DMRequest - also the key in DataMessageProto."""
    category: builtins.str
    """Package name. DMP.2"""
    token: builtins.str
    """The collapsed key, DMP.3"""
    @property
    def app_data(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        global___AppData
    ]:
        """User data + GOOGLE. prefixed special entries, DMP.4"""
    from_trusted_server: builtins.bool
    """Not used."""
    persistent_id: builtins.str
    """Part of the ACK protocol, returned in DataMessageResponse on server side.
    It's part of the key of DMP.
    """
    stream_id: builtins.int
    """In-stream ack. Increments on each message sent - a bit redundant
    Not used in DMP/DMR.
    """
    last_stream_id_received: builtins.int
    reg_id: builtins.str
    """Not used.
    optional string permission = 12;

    Sent by the device shortly after registration.
    """
    device_user_id: builtins.int
    """Not used.
    optional string pkg_signature = 14;
    Not used.
    optional string client_id = 15;

    serial number of the target user, DMP.8
    It is the 'serial number' according to user manager.
    """
    ttl: builtins.int
    """Time to live, in seconds."""
    sent: builtins.int
    """Timestamp ( according to client ) when message was sent by app, in seconds"""
    queued: builtins.int
    """How long has the message been queued before the flush, in seconds.
    This is needed to account for the time difference between server and
    client: server should adjust 'sent' based on its 'receive' time.
    """
    status: builtins.int
    raw_data: builtins.bytes
    """Optional field containing the binary payload of the message."""
    immediate_ack: builtins.bool
    """Not used.
    How long the message was delayed before it was sent, in seconds.
    optional int32 actual_delay = 23;

    If set the server requests immediate ack. Used for important messages and
    for testing.
    """
    def __init__(
        self,
        *,
        id: builtins.str | None = ...,
        to: builtins.str | None = ...,
        category: builtins.str | None = ...,
        token: builtins.str | None = ...,
        app_data: collections.abc.Iterable[global___AppData] | None = ...,
        from_trusted_server: builtins.bool | None = ...,
        persistent_id: builtins.str | None = ...,
        stream_id: builtins.int | None = ...,
        last_stream_id_received: builtins.int | None = ...,
        reg_id: builtins.str | None = ...,
        device_user_id: builtins.int | None = ...,
        ttl: builtins.int | None = ...,
        sent: builtins.int | None = ...,
        queued: builtins.int | None = ...,
        status: builtins.int | None = ...,
        raw_data: builtins.bytes | None = ...,
        immediate_ack: builtins.bool | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "category",
            b"category",
            "device_user_id",
            b"device_user_id",
            "from",
            b"from",
            "from_trusted_server",
            b"from_trusted_server",
            "id",
            b"id",
            "immediate_ack",
            b"immediate_ack",
            "last_stream_id_received",
            b"last_stream_id_received",
            "persistent_id",
            b"persistent_id",
            "queued",
            b"queued",
            "raw_data",
            b"raw_data",
            "reg_id",
            b"reg_id",
            "sent",
            b"sent",
            "status",
            b"status",
            "stream_id",
            b"stream_id",
            "to",
            b"to",
            "token",
            b"token",
            "ttl",
            b"ttl",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "app_data",
            b"app_data",
            "category",
            b"category",
            "device_user_id",
            b"device_user_id",
            "from",
            b"from",
            "from_trusted_server",
            b"from_trusted_server",
            "id",
            b"id",
            "immediate_ack",
            b"immediate_ack",
            "last_stream_id_received",
            b"last_stream_id_received",
            "persistent_id",
            b"persistent_id",
            "queued",
            b"queued",
            "raw_data",
            b"raw_data",
            "reg_id",
            b"reg_id",
            "sent",
            b"sent",
            "status",
            b"status",
            "stream_id",
            b"stream_id",
            "to",
            b"to",
            "token",
            b"token",
            "ttl",
            b"ttl",
        ],
    ) -> None: ...

global___DataMessageStanza = DataMessageStanza

@typing_extensions.final
class StreamAck(google.protobuf.message.Message):
    """*
    Included in IQ with ID 13, sent from client or server after 10 unconfirmed
    messages.
    No last_streamid_received required.  This is included within an IqStanza,
    which includes the last_stream_id_received.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StreamAck = StreamAck

@typing_extensions.final
class SelectiveAck(google.protobuf.message.Message):
    """*
    Included in IQ sent after LoginResponse from server with ID 12.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    @property
    def id(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.str
    ]: ...
    def __init__(
        self,
        *,
        id: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["id", b"id"]
    ) -> None: ...

global___SelectiveAck = SelectiveAck



from __future__ import annotations

import asyncio
import json
import logging
import os
import secrets
import time
import uuid
from base64 import b64encode, urlsafe_b64encode
from dataclasses import dataclass
from typing import Any, Callable

from aiohttp import ClientSession, ClientTimeout
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import ec
from google.protobuf.json_format import MessageToDict, MessageToJson

from .const import (
    AUTH_VERSION,
    FCM_INSTALLATION,
    FCM_REGISTRATION,
    FCM_SEND_URL,
    GCM_CHECKIN_URL,
    GCM_REGISTER_URL,
    GCM_SERVER_KEY_B64,
    SDK_VERSION,
)
from .android_checkin_pb2 import (
    DEVICE_CHROME_BROWSER,
    AndroidCheckinProto,
    ChromeBuildProto,
)
from .checkin_pb2 import (
    AndroidCheckinRequest,
    AndroidCheckinResponse,
)

_logger = logging.getLogger(__name__)


@dataclass
class FcmRegisterConfig:
    project_id: str
    app_id: str
    api_key: str
    messaging_sender_id: str
    bundle_id: str = "receiver.push.com"
    chrome_id: str = "org.chromium.linux"
    chrome_version: str = "94.0.4606.51"
    vapid_key: str | None = GCM_SERVER_KEY_B64
    persistend_ids: list[str] | None = None
    heartbeat_interval_ms: int = 5 * 60 * 1000  # 5 mins

    def __postinit__(self) -> None:
        if self.persistend_ids is None:
            self.persistend_ids = []


class FcmRegister:
    CLIENT_TIMEOUT = ClientTimeout(total=3)

    def __init__(
        self,
        config: FcmRegisterConfig,
        credentials: dict,
        credentials_updated_callback, #: Callable[[dict[str, Any], str], None] ,
        *,
        http_client_session: ClientSession | None = None,
        log_debug_verbose: bool = False,
    ):
        self.config = config
        self.credentials = credentials
        self.credentials_updated_callback = credentials_updated_callback

        self._log_debug_verbose = log_debug_verbose

        self._http_client_session = http_client_session
        self._local_session: ClientSession | None = None

    def _get_checkin_payload(
        self, android_id: int | None = None, security_token: int | None = None
    ) -> AndroidCheckinRequest:
        chrome = ChromeBuildProto()
        chrome.platform = ChromeBuildProto.Platform.PLATFORM_LINUX  # 3
        chrome.chrome_version = self.config.chrome_version
        chrome.channel = ChromeBuildProto.Channel.CHANNEL_STABLE  # 1

        checkin = AndroidCheckinProto()
        checkin.type = DEVICE_CHROME_BROWSER  # 3
        checkin.chrome_build.CopyFrom(chrome)

        payload = AndroidCheckinRequest()
        payload.user_serial_number = 0
        payload.checkin.CopyFrom(checkin)
        payload.version = 3
        if android_id and security_token:
            payload.id = int(android_id)
            payload.security_token = int(security_token)

        return payload

    async def gcm_check_in_and_register(
        self,
    ) -> dict[str, Any] | None:
        options = await self.gcm_check_in()
        if not options:
            raise RuntimeError("Unable to register and check in to gcm")
        gcm_credentials = await self.gcm_register(options)
        return gcm_credentials

    async def gcm_check_in(
        self,
        android_id: int | None = None,
        security_token: int | None = None,
    ) -> dict[str, Any] | None:
        """
        perform check-in request

        android_id, security_token can be provided if we already did the initial
        check-in

        returns dict with android_id, security_token and more
        """

        payload = self._get_checkin_payload(android_id, security_token)

        if self._log_debug_verbose:
            _logger.debug("GCM check in payload:\n%s", payload)

        retries = 3
        acir = None
        content = None
        for try_num in range(retries):
            try:
                async with self._session.post(
                    url=GCM_CHECKIN_URL,
                    headers={"Content-Type": "application/x-protobuf"},
                    data=payload.SerializeToString(),
                    timeout=self.CLIENT_TIMEOUT,
                ) as resp:
                    if resp.status == 200:
                        acir = AndroidCheckinResponse()
                        content = await resp.read()
                        break
                    else:
                        text = await resp.text()
                if acir and content:
                    break
                else:
                    _logger.warning(
                        "GCM checkin failed on attempt %s out "
                        + "of %s with status: %s, %s",
                        try_num + 1,
                        retries,
                        resp.status,
                        text,
                    )
                # retry without android id and security_token
                payload = self._get_checkin_payload()
                await asyncio.sleep(1)
            except Exception as e:
                _logger.warning(
                    "Error during gcm checkin post attempt %s out of %s",
                    try_num + 1,
                    retries,
                    exc_info=e,
                )
                await asyncio.sleep(1)

        if not acir or not content:
            _logger.error("Unable to checkin to gcm after %s retries", retries)
            return None
        acir.ParseFromString(content)

        if self._log_debug_verbose:
            msg = MessageToJson(acir, indent=4)
            _logger.debug("GCM check in response (raw):\n%s", msg)

        return MessageToDict(acir)

    async def gcm_register(
        self,
        options: dict[str, Any],
        retries: int = 4,
    ) -> dict[str, str] | None:
        """
        obtains a gcm token

        app_id: app id as an integer
        retries: number of failed requests before giving up

        returns {"token": "...", "gcm_app_id": 123123, "androidId":123123,
                        "securityToken": 123123}
        """
        # contains android_id, security_token and more
        gcm_app_id = f"wp:{self.config.bundle_id}#{uuid.uuid4()}"
        android_id = options["androidId"]
        security_token = options["securityToken"]

        headers = {
            "Authorization": f"AidLogin {android_id}:{security_token}",
            "Content-Type": "application/x-www-form-urlencoded",
        }
        body = {
            "app": "org.chromium.linux",
            "X-subtype": gcm_app_id,
            "device": android_id,
            "sender": GCM_SERVER_KEY_B64,
        }
        if self._log_debug_verbose:
            _logger.debug("GCM Registration request: %s", body)

        last_error: str | Exception | None = None
        for try_num in range(retries):
            try:
                async with self._session.post(
                    url=GCM_REGISTER_URL,
                    headers=headers,
                    data=body,
                    timeout=self.CLIENT_TIMEOUT,
                ) as resp:
                    response_text = await resp.text()
                if "Error" in response_text:
                    _logger.warning(
                        "GCM register request attempt %s out of %s has failed with %s",
                        try_num + 1,
                        retries,
                        response_text,
                    )
                    last_error = response_text
                    await asyncio.sleep(1)
                    continue
                token = response_text.split("=")[1]

                return {
                    "token": token,
                    "app_id": gcm_app_id,
                    "android_id": android_id,
                    "security_token": security_token,
                }

            except Exception as e:
                last_error = e
                _logger.warning(
                    "Error during gcm auth request attempt %s out of %s",
                    try_num + 1,
                    retries,
                    exc_info=e,
                )
                await asyncio.sleep(1)

        errorstr = f"Unable to complete gcm auth request after {retries} tries"
        if isinstance(last_error, Exception):
            _logger.error(errorstr, exc_info=last_error)
        else:
            errorstr += f", last error was {last_error}"
            _logger.error(errorstr)
        return None

    async def fcm_install_and_register(
        self, gcm_data: dict[str, Any], keys: dict[str, Any]
    ) -> dict[str, Any] | None:
        if installation := await self.fcm_install():
            registration = await self.fcm_register(gcm_data, installation, keys)
            return {
                "registration": registration,
                "installation": installation,
            }
        return None

    async def fcm_install(self) -> dict | None:
        fid = bytearray(secrets.token_bytes(17))
        # Replace the first 4 bits with the FID header 0b0111.
        fid[0] = 0b01110000 + (fid[0] % 0b00010000)
        fid64 = b64encode(fid).decode()

        hb_header = b64encode(
            json.dumps({"heartbeats": [], "version": 2}).encode()
        ).decode()
        headers = {
            "x-firebase-client": hb_header,
            "x-goog-api-key": self.config.api_key,
        }
        payload = {
            "appId": self.config.app_id,
            "authVersion": AUTH_VERSION,
            "fid": fid64,
            "sdkVersion": SDK_VERSION,
        }
        url = FCM_INSTALLATION + f"projects/{self.config.project_id}/installations"
        async with self._session.post(
            url=url,
            headers=headers,
            data=json.dumps(payload),
            timeout=self.CLIENT_TIMEOUT,
        ) as resp:
            if resp.status == 200:
                fcm_install = await resp.json()

                return {
                    "token": fcm_install["authToken"]["token"],
                    "expires_in": int(fcm_install["authToken"]["expiresIn"][:-1:]),
                    "refresh_token": fcm_install["refreshToken"],
                    "fid": fcm_install["fid"],
                    "created_at": time.monotonic(),
                }
            else:
                text = await resp.text()
                _logger.error(
                    "Error during fcm_install: %s ",
                    text,
                )
                return None

    async def fcm_refresh_install_token(self) -> dict | None:
        hb_header = b64encode(
            json.dumps({"heartbeats": [], "version": 2}).encode()
        ).decode()
        if not self.credentials:
            raise RuntimeError("Credentials must be set to refresh install token")
        fcm_refresh_token = self.credentials["fcm"]["installation"]["refresh_token"]

        headers = {
            "Authorization": f"{AUTH_VERSION} {fcm_refresh_token}",
            "x-firebase-client": hb_header,
            "x-goog-api-key": self.config.api_key,
        }
        payload = {
            "installation": {
                "sdkVersion": SDK_VERSION,
                "appId": self.config.app_id,
            }
        }
        url = (
            FCM_INSTALLATION + f"projects/{self.config.project_id}/"
            "installations/{fid}/authTokens:generate"
        )
        async with self._session.post(
            url=url,
            headers=headers,
            data=json.dumps(payload),
            timeout=self.CLIENT_TIMEOUT,
        ) as resp:
            if resp.status == 200:
                fcm_refresh = await resp.json()
                return {
                    "token": fcm_refresh["token"],
                    "expires_in": int(fcm_refresh["expiresIn"][:-1:]),
                    "created_at": time.monotonic(),
                }
            else:
                text = await resp.text()
                _logger.error(
                    "Error during fcm_refresh_install_token: %s ",
                    text,
                )
                return None

    def generate_keys(self) -> dict:
        private_key = ec.generate_private_key(ec.SECP256R1())
        public_key = private_key.public_key()

        serialized_private = private_key.private_bytes(
            encoding=serialization.Encoding.DER,  # asn1
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        )
        serialized_public = public_key.public_bytes(
            encoding=serialization.Encoding.DER,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )

        return {
            "public": urlsafe_b64encode(serialized_public[26:]).decode(
                "ascii"
            ),  # urlsafe_base64(serialized_public[26:]),
            "private": urlsafe_b64encode(serialized_private).decode("ascii"),
            "secret": urlsafe_b64encode(os.urandom(16)).decode("ascii"),
        }

    async def fcm_register(
        self,
        gcm_data: dict,
        installation: dict,
        keys: dict,
        retries: int = 4,
    ) -> dict[str, Any] | None:
        headers = {
            "x-goog-api-key": self.config.api_key,
            "x-goog-firebase-installations-auth": installation["token"],
        }
        # If vapid_key is the default do not send it here or it will error
        vapid_key = (
            self.config.vapid_key
            if self.config.vapid_key != GCM_SERVER_KEY_B64
            else None
        )
        payload = {
            "web": {
                "applicationPubKey": vapid_key,
                "auth": keys["secret"],
                "endpoint": FCM_SEND_URL + gcm_data["token"],
                "p256dh": keys["public"],
            }
        }
        url = FCM_REGISTRATION + f"projects/{self.config.project_id}/registrations"
        if self._log_debug_verbose:
            _logger.debug("FCM registration data: %s", payload)

        for try_num in range(retries):
            try:
                async with self._session.post(
                    url=url,
                    headers=headers,
                    data=json.dumps(payload),
                    timeout=self.CLIENT_TIMEOUT,
                ) as resp:
                    if resp.status == 200:
                        fcm = await resp.json()
                        return fcm
                    else:
                        text = await resp.text()
                        _logger.error(  # pylint: disable=duplicate-code
                            "Error during fmc register request "
                            "attempt %s out of %s: %s",
                            try_num + 1,
                            retries,
                            text,
                        )

            except Exception as e:
                _logger.error(  # pylint: disable=duplicate-code
                    "Error during fmc register request attempt %s out of %s",
                    try_num + 1,
                    retries,
                    exc_info=e,
                )
                await asyncio.sleep(1)
        return None

    async def checkin_or_register(self, fcmCredentialsFileName) -> dict[str, Any]:
        """Check in if you have credentials otherwise register as a new client.

        :param sender_id: sender id identifying push service you are connecting to.
        :param app_id: identifier for your application.
        :return: The FCM token which is used to identify you with the push end
            point application.
        """
        if self.credentials:
            gcm_response = await self.gcm_check_in(
                self.credentials["gcm"]["android_id"],
                self.credentials["gcm"]["security_token"],
            )
            if gcm_response:
                return self.credentials

        self.credentials = await self.register()
        if True: #self.credentials_updated_callback != None:
            await self.credentials_updated_callback(self.credentials, fcmCredentialsFileName)

        return self.credentials

    async def register(self) -> dict:
        """Register gcm and fcm tokens for sender_id.
            Typically you would
            call checkin instead of register which does not do a full registration
            if credentials are present

        :param sender_id: sender id identifying push service you are connecting to.
        :param app_id: identifier for your application.
        :return: The dict containing all credentials.
        """

        keys = self.generate_keys()

        gcm_data = await self.gcm_check_in_and_register()
        if gcm_data is None:
            raise RuntimeError(
                "Unable to establish subscription with Google Cloud Messaging."
            )
        self._log_verbose("GCM subscription: %s", gcm_data)

        fcm_data = await self.fcm_install_and_register(gcm_data, keys)
        if not fcm_data:
            raise RuntimeError("Unable to register with fcm")
        self._log_verbose("FCM registration: %s", fcm_data)
        res: dict[str, Any] = {
            "keys": keys,
            "gcm": gcm_data,
            "fcm": fcm_data,
            "config": {
                "bundle_id": self.config.bundle_id,
                "project_id": self.config.project_id,
                "vapid_key": self.config.vapid_key,
            },
        }
        self._log_verbose("Credential: %s", res)
        _logger.info("Registered with FCM")
        return res

    def _log_verbose(self, msg: str, *args: object) -> None:
        if self._log_debug_verbose:
            _logger.debug(msg, *args)

    @property
    def _session(self) -> ClientSession:
        if self._http_client_session:
            return self._http_client_session
        if self._local_session is None:
            self._local_session = ClientSession()
        return self._local_session

    async def close(self) -> None:
        """Close aiohttp session."""
        session = self._local_session
        self._local_session = None
        if session:
            await session.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: android_checkin.proto
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x15\x61ndroid_checkin.proto\x12\rcheckin_proto"\x8a\x03\n\x10\x43hromeBuildProto\x12:\n\x08platform\x18\x01 \x01(\x0e\x32(.checkin_proto.ChromeBuildProto.Platform\x12\x16\n\x0e\x63hrome_version\x18\x02 \x01(\t\x12\x38\n\x07\x63hannel\x18\x03 \x01(\x0e\x32\'.checkin_proto.ChromeBuildProto.Channel"}\n\x08Platform\x12\x10\n\x0cPLATFORM_WIN\x10\x01\x12\x10\n\x0cPLATFORM_MAC\x10\x02\x12\x12\n\x0ePLATFORM_LINUX\x10\x03\x12\x11\n\rPLATFORM_CROS\x10\x04\x12\x10\n\x0cPLATFORM_IOS\x10\x05\x12\x14\n\x10PLATFORM_ANDROID\x10\x06"i\n\x07\x43hannel\x12\x12\n\x0e\x43HANNEL_STABLE\x10\x01\x12\x10\n\x0c\x43HANNEL_BETA\x10\x02\x12\x0f\n\x0b\x43HANNEL_DEV\x10\x03\x12\x12\n\x0e\x43HANNEL_CANARY\x10\x04\x12\x13\n\x0f\x43HANNEL_UNKNOWN\x10\x05"\xf6\x01\n\x13\x41ndroidCheckinProto\x12\x19\n\x11last_checkin_msec\x18\x02 \x01(\x03\x12\x15\n\rcell_operator\x18\x06 \x01(\t\x12\x14\n\x0csim_operator\x18\x07 \x01(\t\x12\x0f\n\x07roaming\x18\x08 \x01(\t\x12\x13\n\x0buser_number\x18\t \x01(\x05\x12:\n\x04type\x18\x0c \x01(\x0e\x32\x19.checkin_proto.DeviceType:\x11\x44\x45VICE_ANDROID_OS\x12\x35\n\x0c\x63hrome_build\x18\r \x01(\x0b\x32\x1f.checkin_proto.ChromeBuildProto*g\n\nDeviceType\x12\x15\n\x11\x44\x45VICE_ANDROID_OS\x10\x01\x12\x11\n\rDEVICE_IOS_OS\x10\x02\x12\x19\n\x15\x44\x45VICE_CHROME_BROWSER\x10\x03\x12\x14\n\x10\x44\x45VICE_CHROME_OS\x10\x04\x42\x02H\x03'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "android_checkin_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals["DESCRIPTOR"]._serialized_options = b"H\003"
    _globals["_DEVICETYPE"]._serialized_start = 686
    _globals["_DEVICETYPE"]._serialized_end = 789
    _globals["_CHROMEBUILDPROTO"]._serialized_start = 41
    _globals["_CHROMEBUILDPROTO"]._serialized_end = 435
    _globals["_CHROMEBUILDPROTO_PLATFORM"]._serialized_start = 203
    _globals["_CHROMEBUILDPROTO_PLATFORM"]._serialized_end = 328
    _globals["_CHROMEBUILDPROTO_CHANNEL"]._serialized_start = 330
    _globals["_CHROMEBUILDPROTO_CHANNEL"]._serialized_end = 435
    _globals["_ANDROIDCHECKINPROTO"]._serialized_start = 438
    _globals["_ANDROIDCHECKINPROTO"]._serialized_end = 684
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
""" Sample program to export the trip statistics as csv file"""
import asyncio
import logging
import inspect
import sys
import os
import json
import pandas as pd
from aiohttp import ClientSession
from datetime import datetime

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)

try:
    from pycupra import Connection
except ModuleNotFoundError as e:
    print(f"Unable to import library: {e}")
    sys.exit(1)

logging.basicConfig(level=logging.WARN)
_LOGGER = logging.getLogger(__name__)
BRAND = 'cupra' # or 'seat' (Default value if no brand is provided via credentials file)

PRINTRESPONSE = True
INTERVAL = 5
TOKEN_FILE_NAME_AND_PATH='./pycupra_token.json'
CREDENTIALS_FILE_NAME_AND_PATH='./pycupra_credentials.json'

def readCredentialsFile():
    try:
        with open(CREDENTIALS_FILE_NAME_AND_PATH, "r") as f:
            credentialsString=f.read()
        credentials=json.loads(credentialsString)
        return credentials
    except:
        _LOGGER.info('readCredentialsFile not successful. Perhaps no credentials file present.')
        return None

def exportToCSV(vehicle, csvFileName, dataType='short'):
    df= pd.DataFrame(vehicle._states['tripstatistics'][dataType])
    _LOGGER.debug('Exporting trip data to csv')
    df.to_csv(csvFileName)
    return True

async def main():
    """Main method."""
    print('')
    print('######################################################')
    print('#               Reading credentials file             #')
    print('######################################################')
    credentials= readCredentialsFile()
    if credentials==None or credentials.get('username','')=='' or (credentials.get('password','')==''):
        _LOGGER.warning('Can not use the credentials read from the credentials file.')
        raise
    if credentials.get('brand','')!='':
        BRAND = credentials.get('brand','')
        print('Read brand from the credentials file.')
    else:
        print('No brand found in the credentials file. Using the default value.')
    print(f'Now working with brand={BRAND}')
    async with ClientSession(headers={'Connection': 'keep-alive'}) as session:
        print('')
        print('######################################################')
        print('# Logging on to ola.prod.code.seat.cloud.vwgroup.com #')
        print('######################################################')
        print(f"Initiating new session to Cupra/Seat Cloud with {credentials.get('username')} as username")
        connection = Connection(session, BRAND, credentials.get('username'), credentials.get('password'), PRINTRESPONSE, nightlyUpdateReduction=False, anonymise=True, tripStatisticsStartDate='1970-01-01')
        print("Attempting to login to the Seat Cloud service")
        if await connection.doLogin(tokenFile=TOKEN_FILE_NAME_AND_PATH, apiKey=credentials.get('apiKey',None)):
            print('Login or token refresh success!')
            print(datetime.now())
            print('Fetching user information for account.')
            await connection.get_userData()
            print(f"\tName: {connection._userData.get('name','')}")
            print(f"\tNickname: {connection._userData.get('nickname','')}")
            print(f"\tEmail: {connection._userData.get('email','')}")
            print(f"\tPicture: {connection._userData.get('picture','')}")
            print("")
            print('Fetching vehicles associated with account.')
            await connection.get_vehicles()

            print('')
            print('########################################')
            print('#          Vehicles discovered         #')
            print('########################################')
            for vehicle in connection.vehicles:
                print(f"\tVIN: {vehicle.vin}")
                print(f"\tModel: {vehicle.model}")
                print(f"\tManufactured: {vehicle.model_year}")
                print(f"\tConnect service deactivated: {vehicle.deactivated}")
                print("")
                if vehicle.is_nickname_supported: print(f"\tNickname: {vehicle.nickname}")
        else:
            return False

        for vehicle in connection.vehicles:
            txt = vehicle.vin
            print('########################################')
            print('#     Export driving data to csv       #')
            print(txt.center(40, '#'))
            exportToCSV(vehicle, credentials.get('csvFileName','./drivingData.csv'), 'short') # possible value: short/cyclic
            print('')
            print('Export of driving data to csv complete')
    sys.exit(1)

if __name__ == "__main__":
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    loop.run_until_complete(main())




# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: checkin.proto
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\rcheckin.proto\x12\rcheckin_proto\x1a\x15\x61ndroid_checkin.proto"/\n\x10GservicesSetting\x12\x0c\n\x04name\x18\x01 \x02(\x0c\x12\r\n\x05value\x18\x02 \x02(\x0c"\xcb\x03\n\x15\x41ndroidCheckinRequest\x12\x0c\n\x04imei\x18\x01 \x01(\t\x12\x0c\n\x04meid\x18\n \x01(\t\x12\x10\n\x08mac_addr\x18\t \x03(\t\x12\x15\n\rmac_addr_type\x18\x13 \x03(\t\x12\x15\n\rserial_number\x18\x10 \x01(\t\x12\x0b\n\x03\x65sn\x18\x11 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\x03\x12\x12\n\nlogging_id\x18\x07 \x01(\x03\x12\x0e\n\x06\x64igest\x18\x03 \x01(\t\x12\x0e\n\x06locale\x18\x06 \x01(\t\x12\x33\n\x07\x63heckin\x18\x04 \x02(\x0b\x32".checkin_proto.AndroidCheckinProto\x12\x15\n\rdesired_build\x18\x05 \x01(\t\x12\x16\n\x0emarket_checkin\x18\x08 \x01(\t\x12\x16\n\x0e\x61\x63\x63ount_cookie\x18\x0b \x03(\t\x12\x11\n\ttime_zone\x18\x0c \x01(\t\x12\x16\n\x0esecurity_token\x18\r \x01(\x06\x12\x0f\n\x07version\x18\x0e \x01(\x05\x12\x10\n\x08ota_cert\x18\x0f \x03(\t\x12\x10\n\x08\x66ragment\x18\x14 \x01(\x05\x12\x11\n\tuser_name\x18\x15 \x01(\t\x12\x1a\n\x12user_serial_number\x18\x16 \x01(\x05"\x83\x02\n\x16\x41ndroidCheckinResponse\x12\x10\n\x08stats_ok\x18\x01 \x02(\x08\x12\x11\n\ttime_msec\x18\x03 \x01(\x03\x12\x0e\n\x06\x64igest\x18\x04 \x01(\t\x12\x15\n\rsettings_diff\x18\t \x01(\x08\x12\x16\n\x0e\x64\x65lete_setting\x18\n \x03(\t\x12\x30\n\x07setting\x18\x05 \x03(\x0b\x32\x1f.checkin_proto.GservicesSetting\x12\x11\n\tmarket_ok\x18\x06 \x01(\x08\x12\x12\n\nandroid_id\x18\x07 \x01(\x06\x12\x16\n\x0esecurity_token\x18\x08 \x01(\x06\x12\x14\n\x0cversion_info\x18\x0b \x01(\tB\x02H\x03'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "checkin_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals["DESCRIPTOR"]._serialized_options = b"H\003"
    _globals["_GSERVICESSETTING"]._serialized_start = 55
    _globals["_GSERVICESSETTING"]._serialized_end = 102
    _globals["_ANDROIDCHECKINREQUEST"]._serialized_start = 105
    _globals["_ANDROIDCHECKINREQUEST"]._serialized_end = 564
    _globals["_ANDROIDCHECKINRESPONSE"]._serialized_start = 567
    _globals["_ANDROIDCHECKINRESPONSE"]._serialized_end = 826
# @@protoc_insertion_point(module_scope)
